var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { action, computed, observable, toJS } from 'mobx';
import { ArrayElementNotFoundError, ArrayIndexNotFoundError } from './Errors';
let ArrayStore = /** @class */ (() => {
    class ArrayStore {
        constructor(items = []) {
            this._items = new Array();
            this.insert = this.insert.bind(this);
            this.add = this.add.bind(this);
            this.at = this.at.bind(this);
            this.remove = this.remove.bind(this);
            this.removeAt = this.removeAt.bind(this);
            this.splice = this.splice.bind(this);
            this.clear = this.clear.bind(this);
            this.some = this.some.bind(this);
            this.find = this.find.bind(this);
            this.findIndex = this.findIndex.bind(this);
            this.update = this.update.bind(this);
            this.updateByIndex = this.updateByIndex.bind(this);
            this.setItems = this.setItems.bind(this);
            this.reset = this.reset.bind(this);
            this.toggle = this.toggle.bind(this);
            this._defaultValue = items;
            this._items = items;
        }
        get isDefault() {
            const a = this._defaultValue;
            const b = this.items;
            if (a === b) {
                return true;
            }
            if (a.length !== b.length) {
                return false;
            }
            for (var i = 0; i < a.length; ++i) {
                if (a[i] !== b[i]) {
                    return false;
                }
            }
            return true;
        }
        get items() {
            return this._items;
        }
        get count() {
            return this._items.length;
        }
        get isEmpty() {
            return this._items.length === 0;
        }
        get first() {
            return this._items[0] || undefined;
        }
        get last() {
            return this.items[this.items.length - 1] || undefined;
        }
        add(...item) {
            this._items.push(...item);
            return this;
        }
        at(index) {
            return toJS(this._items)[index] || undefined;
        }
        setItems(items) {
            this._items = items;
            return this;
        }
        insert(index, item) {
            this._items.splice(index, 0, item);
            return this;
        }
        remove(item) {
            const itemIndex = this._items.findIndex(x => x === item);
            const itemWasFound = itemIndex > -1;
            if (itemWasFound) {
                this.removeAt(itemIndex);
                return true;
            }
            return false;
        }
        removeAt(index) {
            if (this.isEmpty
                || this._items.length < index) {
                return false;
            }
            this._items.splice(index, 1);
            return true;
        }
        filter(predicate) {
            this.setItems(this._items.filter(predicate));
            return this;
        }
        splice(index, count) {
            this._items.splice(index, count);
            return this;
        }
        some(item) {
            return this._items.some(x => item === x);
        }
        find(predicate, thisArg) {
            return this._items.find(predicate, thisArg);
        }
        findIndex(predicate, thisArg) {
            return this._items.findIndex(predicate, thisArg);
        }
        updateByIndex(index, value) {
            const items = toJS(this._items);
            if (items[index] === undefined) {
                throw new ArrayIndexNotFoundError(index);
            }
            this._items[index] = value;
            return this;
        }
        update(predicate, value) {
            const index = this.findIndex(predicate);
            if (index === -1) {
                throw new ArrayElementNotFoundError();
            }
            this._items[index] = value;
            return this;
        }
        reset() {
            this.setItems(this._defaultValue);
            return this;
        }
        clear() {
            this.setItems([]);
            return this;
        }
        toggle(item) {
            this.some(item)
                ? this.remove(item)
                : this.add(item);
            return this;
        }
    }
    __decorate([
        observable
    ], ArrayStore.prototype, "_items", void 0);
    __decorate([
        computed
    ], ArrayStore.prototype, "isDefault", null);
    __decorate([
        computed
    ], ArrayStore.prototype, "items", null);
    __decorate([
        computed
    ], ArrayStore.prototype, "count", null);
    __decorate([
        computed
    ], ArrayStore.prototype, "isEmpty", null);
    __decorate([
        computed
    ], ArrayStore.prototype, "first", null);
    __decorate([
        computed
    ], ArrayStore.prototype, "last", null);
    __decorate([
        action
    ], ArrayStore.prototype, "add", null);
    __decorate([
        action
    ], ArrayStore.prototype, "setItems", null);
    __decorate([
        action
    ], ArrayStore.prototype, "insert", null);
    __decorate([
        action
    ], ArrayStore.prototype, "removeAt", null);
    __decorate([
        action
    ], ArrayStore.prototype, "splice", null);
    __decorate([
        action
    ], ArrayStore.prototype, "updateByIndex", null);
    __decorate([
        action
    ], ArrayStore.prototype, "update", null);
    return ArrayStore;
})();
export { ArrayStore };
//# sourceMappingURL=ArrayStore.js.map