var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { OperationStateStore } from '../OperationState';
import { ObjectStore } from '../Object';
import { WrongNumberOfRetriesError } from './Errors';
import { computed } from 'mobx';
let AsyncOperationStore = /** @class */ (() => {
    class AsyncOperationStore {
        constructor(operation, options = {}) {
            this.state = new OperationStateStore();
            this._error = new ObjectStore();
            this._result = new ObjectStore();
            this.run = this.run.bind(this);
            this.reset = this.reset.bind(this);
            this._operation = operation;
            this._options = Object.assign({
                preventDoubleSubmission: true,
                numberOfRetries: 0,
            }, options);
            if (this._options.numberOfRetries < 0) {
                throw new WrongNumberOfRetriesError();
            }
            this.execute = this.execute.bind(this);
        }
        get error() {
            return this._error.value;
        }
        get result() {
            return this._result.value;
        }
        execute(params) {
            const state = this.state;
            this._params = params;
            if (this._options.preventDoubleSubmission
                && this._resultPromise
                && state.isRunning) {
                return this._resultPromise;
            }
            state.start();
            this._resultPromise = new Promise(this.run);
            return this._resultPromise;
        }
        normalizeError(e) {
            return typeof e === 'string'
                ? new Error(e)
                : e;
        }
        async run(resolve, reject) {
            const state = this.state;
            let tries = this._options.numberOfRetries + 1;
            while (tries-- > 0) {
                try {
                    const result = await this._operation(this._params);
                    this._result.set(result);
                    resolve(result);
                    state.done();
                    break;
                }
                catch (e) {
                    if (tries > 0) {
                        continue;
                    }
                    const error = this.normalizeError(e);
                    this._error.set(error);
                    state.fail();
                    reject(error);
                }
            }
        }
        async abort() {
            if (this._options.abort) {
                await this._options.abort();
            }
            this.state.abort();
        }
        reset() {
            this.state.idle();
        }
    }
    __decorate([
        computed
    ], AsyncOperationStore.prototype, "error", null);
    __decorate([
        computed
    ], AsyncOperationStore.prototype, "result", null);
    return AsyncOperationStore;
})();
export { AsyncOperationStore };
//# sourceMappingURL=AsyncOperationStore.js.map