export class DisposerStore {
    constructor() {
        this._disposers = [];
        this.push = this.push.bind(this);
        this.dispose = this.dispose.bind(this);
    }
    push(...items) {
        this._disposers = items.map(method => ({
            method: method,
            isDisposed: false,
        }));
    }
    async dispose() {
        try {
            const methods = this._disposers.map(x => {
                const promise = x.method();
                if (promise instanceof Promise) {
                    promise.then(() => {
                        x.isDisposed = true;
                    });
                }
                else {
                    x.isDisposed = true;
                }
                return promise;
            });
            await Promise.all(methods);
        }
        finally {
            this._disposers = this._disposers.filter(x => !x.isDisposed);
        }
    }
}
//# sourceMappingURL=DisposerStore.js.map