var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { ValueStore } from '../Value';
import { computed } from 'mobx';
let TimerStore = /** @class */ (() => {
    class TimerStore {
        constructor(operation, timout, options = {}) {
            this._timer = new ValueStore();
            this._startedAt = new ValueStore();
            this._doneAt = new ValueStore();
            this._canceledAt = new ValueStore();
            this.start = this.start.bind(this);
            this.stop = this.stop.bind(this);
            this.cancel = this.cancel.bind(this);
            this.scheduleNextLoop = this.scheduleNextLoop.bind(this);
            this._operation = operation;
            this._timeout = timout;
            this._options = Object.assign({
            // add default options if required
            }, options);
        }
        get startedAt() {
            return this._startedAt.value;
        }
        get doneAt() {
            return this._doneAt.value;
        }
        get canceledAt() {
            return this._canceledAt.value;
        }
        get isStarted() {
            return (!this._startedAt.isDefault
                && !this.isCanceled);
        }
        get isDone() {
            return (!this._doneAt.isDefault
                && !this.isCanceled);
        }
        get isCanceled() {
            return !this._canceledAt.isDefault;
        }
        get isRunning() {
            return (this.isStarted
                && !this.isCanceled
                && !this.isDone);
        }
        async start() {
            if (this.isRunning) {
                return;
            }
            this._doneAt.reset();
            this._canceledAt.reset();
            this._startedAt.set(new Date().getTime());
            await this._operation.execute();
            this._doneAt.set(new Date().getTime());
            this.scheduleNextLoop();
        }
        scheduleNextLoop() {
            if (this._options.delayBetweenLoops) {
                this.clearInterval();
                this._timer.set(setInterval(this.start, this._timeout));
            }
        }
        async cancel() {
            this.clearInterval();
            if (this._operation.abort) {
                await this._operation.abort();
            }
            this._canceledAt.set(new Date().getTime());
        }
        clearInterval() {
            clearInterval(this._timer.value);
            this._timer.reset();
        }
        async stop() {
            await this.cancel();
        }
    }
    __decorate([
        computed
    ], TimerStore.prototype, "startedAt", null);
    __decorate([
        computed
    ], TimerStore.prototype, "doneAt", null);
    __decorate([
        computed
    ], TimerStore.prototype, "canceledAt", null);
    __decorate([
        computed
    ], TimerStore.prototype, "isStarted", null);
    __decorate([
        computed
    ], TimerStore.prototype, "isDone", null);
    __decorate([
        computed
    ], TimerStore.prototype, "isCanceled", null);
    __decorate([
        computed
    ], TimerStore.prototype, "isRunning", null);
    return TimerStore;
})();
export { TimerStore };
//# sourceMappingURL=TimerStore.js.map